import pygame
import pygame.cdrom

CD_PLAYMODE_IDLE = 0
CD_PLAYMODE_STOP = 1
CD_PLAYMODE_NEXT = 2
CD_PLAYMODE_CONTINUOUS = 3
CD_PLAYMODE_PREVIEW = 4
CD_PLAYMODE_PREVIEW2 = 5
CD_PLAYMODE_AGAIN = 6
CD_PLAYMODE_LAST= 7

preview_start = 5
preview_end = 10

class CD:
    def __init__(self, n):
        self.cd = pygame.cdrom.CD(n)
        self.cd.init()
        self.cd.stop()
        self.track = -1
        self.playmode = CD_PLAYMODE_STOP

    def SetPlayMode(self, mode):
        assert mode >= CD_PLAYMODE_IDLE and mode <= CD_PLAYMODE_LAST
        self.playmode = mode

    def IsPreviewing(self):
        return self.playmode == CD_PLAYMODE_PREVIEW or self.playmode == CD_PLAYMODE_PREVIEW2

    def NextTrack(self):
        self.track = self.track + 1
        if self.track == self.cd.get_numtracks():
            self.track = 0

    def ExecutePlayMode(self):      # call this function periodically
        assert self.playmode >= CD_PLAYMODE_IDLE and self.playmode <= CD_PLAYMODE_LAST

        if self.cd.get_empty():  # no CD
            self.track = -1
            self.playmode = CD_PLAYMODE_IDLE
            return;

        if self.playmode == CD_PLAYMODE_IDLE:
            return
            
        elif self.playmode == CD_PLAYMODE_STOP:
            if self.cd.get_busy():
                self.Stop()
            self.playmode = CD_PLAYMODE_IDLE
            
        elif self.playmode == CD_PLAYMODE_NEXT:
            self.NextTrack()
            self.Play()
            self.playmode = CD_PLAYMODE_IDLE

        elif self.playmode == CD_PLAYMODE_CONTINUOUS and not self.cd.get_busy():
            self.NextTrack()
            self.Play()

        elif self.playmode == CD_PLAYMODE_PREVIEW:
            self.Stop()
            self.NextTrack()
            self.Play(preview=True)
            self.playmode = CD_PLAYMODE_PREVIEW2

        elif self.playmode == CD_PLAYMODE_PREVIEW2 and not self.cd.get_busy():
            self.NextTrack()
            self.Play(preview=True)

        elif self.playmode == CD_PLAYMODE_AGAIN:
            if  self.track != -1:
                self.Play()
            self.playmode = CD_PLAYMODE_IDLE

    def Stop(self):
        self.cd.stop()

    def IsPlaying(self):
        return self.cd.get_busy()

    def Play(self, preview=False):
        if self.cd.get_track_audio(self.track):
            if preview:
                self.cd.play(self.track, preview_start, preview_end)
            else:
                self.cd.play(self.track)
